/* 
   dreamcast fighting v0.1.5: relative rotation(dodging)

 by
 jimbo
 Dan Potter and Jordan DeLong of Cryptic Allusion
 Andrew K.
 Marcus Comstedt

 dependencies:
  - dcf_fighter.c
  - dcf_control.c

   something to remember: these libs use negative radians in glRotatef()
    relative to my linux/NVidia GL libs.


   using
   KallistiOS 1.1.5

   adapted from
   gltest.c
   (c)2001 Dan Potter
*/

#include <kos.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include <math.h>
#include <stdlib.h>

#include "dcf_fighter.c"

/*** for interpoaltion : a move queue ***/
#define HISTORY_SIZE 32
#define FUTURE_SIZE 64




#define MOVE_AGO( x ) change_history[ (change_pointer - x + HISTORY_SIZE) % HISTORY_SIZE ]
#define TIME_OF( x ) p->future_lists[0][ ( p->future_pointer + x + FUTURE_SIZE) % FUTURE_SIZE ]
#define STATE_OF( x ) p->future_lists[1][ ( p->future_pointer + x + FUTURE_SIZE) % FUTURE_SIZE ]
#define OFFSET_HACK 4

/*** 0.1.2: state variables for combat! ***/
#define FIGHT_STAND   100
#define FIGHT_MOVE    95

/*** above 100 state represents power ***/
#define FIGHT_ATTACK  132
#define FIGHT_PUNCH   132
#define FIGHT_KICK    132
#define MAX_POWER     200
#define FIGHT_THROW   138

#define FIGHT_PARRY   105
#define FIGHT_BLOCK   110
#define FIGHT_COUNTER 115

/*** below 100 state represents endurance... ***/
#define FIGHT_HIT     90
#define FIGHT_STUNNED 80
#define FIGHT_STAGGER 70
#define FIGHT_FALLING 30
#define FIGHT_KNEEL   20
#define FIGHT_DOWN    10

/*** player structure ***/
typedef struct {
  double future_lists[ NUMBER_PARAMETERS + OFFSET_HACK ][ FUTURE_SIZE ];
  int lists_in_future;
  int future_pointer;
  int reverse;
  double *params;

  /*** these are for Dan Potter's button-hold logic ***/
  int trans;
  int trans2;
  int transa;
  int transb;
  int transx;
  int transy;
  int transyb;
  int transab;
  int transltrig;
  int transrtrig;
  
  /*** 0.1.2: state variables for combat! ***/
  int fighting_state;
  int default_state;
  int personal_rotation;

  void *opponent;
} fighter_t;


fighter_t *player1, *player2;




/*** for input history ***/
int controller_history[ HISTORY_SIZE ];
int history_pointer = 0;
int change_history[ HISTORY_SIZE ];
int change_pointer = 0;

/*** need another of above for player 2 ***/

/*** for view ***/
double view_rot = 0.0;
double view_roty = 0.0;
/*** for manipulation ***/
int chosen_param = 0;
/*** for interpolation ***/
double t;
double timestep = 0.02;
/*** for combat ***/
float fighting_distance = 7.0;
/*** dont let you get closer ***/
float minimum_fighting_distance = 1.0;
//float fighting_perpendicular = 0.0;
float fighting_rotation = 0.0;
float fighting_displacement[2] = {0.0,0.0};
/*** how close you need to be to attack ***/
float fighting_range = 3.0;


/* the function version 
 *  v0.1.2 contains fighting state
 *
 */
void add_anchor( fighter_t *p, double x, double *y, int state ) {
  int i;
  /*** add current position as an anchor if list empty ***/
  if ( p->lists_in_future == 0 ) { 
    TIME_OF( 0 ) = 0.0; 
    for ( i=OFFSET_HACK; i< NUMBER_PARAMETERS + OFFSET_HACK; i++ ) { 
      p->future_lists[i][  (p->future_pointer + p->lists_in_future ) % FUTURE_SIZE ] 
	= p->params[ i-OFFSET_HACK ];
    } 
    p->lists_in_future++; 
  }
  /*** if buffer not full ***/
  if ( p->lists_in_future < FUTURE_SIZE ) { 
    TIME_OF( p->lists_in_future ) = x ;
    STATE_OF( p->lists_in_future ) = state ;

    if ( TIME_OF( p->lists_in_future - 1) > 0.0 ) { 
      TIME_OF( p->lists_in_future ) += TIME_OF( p->lists_in_future - 1 ); 
    } 
    for ( i=OFFSET_HACK; i< NUMBER_PARAMETERS + OFFSET_HACK; i++ ) { 
      p->future_lists[i][  (p->future_pointer + p->lists_in_future ) % FUTURE_SIZE ] 
	  = y [ p->reverse*NUMBER_PARAMETERS + i-OFFSET_HACK]; 
    } 
    p->lists_in_future++; 
  } 

}


/*** dcf_control.c refers to some(all?) of the above globals ***/
#include "dcf_control.c"





/*** Dan Potter's code ***/
/* Load a texture using pcx_load_texture and glKosTex2D */
void loadtxr(const char *fn, GLuint *txr) {
	uint32 txaddr, w, h;
	if (pcx_load_texture(fn, 1, 0, &txaddr, &w, &h) < 0) {
		printf("can't load %s\n", fn);
		return;
	}

	glGenTextures(1, txr);
	glBindTexture(GL_TEXTURE_2D, *txr);
	glKosTex2D(GL_RGB565_TWID, w, h, txaddr);
}



extern uint8 romdisk[];

int main(int argc, char **argv) {
  int i;
  int players[2][2];
  int player_count = 0;

  /*** nedd this for my #define macros ***/
  fighter_t *p;


  /*** Dan Potter's variables ***/
  cont_cond_t cond, cond2;
  uint8	c, c2;
  GLuint texture;

  /*** Dan Potter's code ***/
  /* Initialize KOS */
  kos_init_all(THD_ENABLE | IRQ_ENABLE | TA_ENABLE, romdisk);
  /* Get basic stuff initialized */
  glKosInit();
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(30.0f, 640.0f / 480.0f, 0.01f, 1000.0f);
  glMatrixMode(GL_MODELVIEW);
  glEnable(GL_TEXTURE_2D);
  glEnable(GL_KOS_AUTO_UV);
  
  /* Load a texture */
  //loadtxr("/rd/crate.pcx", &texture);
  //loadtxr("/rd/brain.pcx", &texture);
  //loadtxr("/rd/roots.pcx", &texture);
  // loadtxr("/rd/bluedragon.pcx", &texture);
  // loadtxr("/rd/sunrise.pcx", &texture);
  // loadtxr("/rd/splash.pcx", &texture);
  loadtxr("/rd/barbecue.pcx", &texture);
  // loadtxr("/rd/skull.pcx", &texture);
  printf("texture is %08x\n", texture);


  /*** jimbo ***/
  player1 = malloc( sizeof( fighter_t ) );
  player2 = malloc( sizeof( fighter_t ) );

  player1->opponent = player2;
  player2->opponent = player1;

  player1->lists_in_future = 0;
  player1->future_pointer = 0;
  player1->reverse = 0;
  player1->params = p1;

  player1->trans = 0;
  player1->trans2 = 0;
  player1->transa = 0;
  player1->transb = 0;
  player1->transx = 0;
  player1->transy = 0;
  player1->transyb = 0;
  player1->transab = 0;
  player1->transltrig = 0;
  player1->transrtrig = 0;

  player1->fighting_state = FIGHT_STAND;
  player1->default_state = FIGHT_STAND;
  player1->personal_rotation = 0.0;


  player2->lists_in_future = 0;
  player2->future_pointer = 0;
  player2->reverse = 1;
  player2->params = p2;
  player2->trans = 0;
  player2->trans2 = 0;
  player2->transa = 0;
  player2->transb = 0;
  player2->transx = 0;
  player2->transy = 0;
  player2->transyb = 0;
  player2->transab = 0;
  player2->transltrig = 0;
  player2->transrtrig = 0;

  player2->fighting_state = FIGHT_STAND;
  player2->default_state = FIGHT_STAND;
  player2->personal_rotation = 0.0;


  /*** quick init of arrays ***/
  for ( i=0; i<HISTORY_SIZE; i++ ) {
    controller_history[i] = change_history[i] = 65535; /*** means no input ***/
  }


  for ( i=0; i<NUMBER_PARAMETERS; i++ ) {
    player1->params[i] = left_fighting_stance[i];
    player2->params[i] = twohand_push[i];
  }

  for ( i=0; i<FUTURE_SIZE; i++ ) {
    player1->future_lists[0][i] = 0.0;
    player2->future_lists[0][i] = 0.0;
  }


  /*** set up all the reverse moves (one time at init saves us a loop) ***/
  fill_reverse( left_jab );
  fill_reverse( left_jab_kick );
  fill_reverse( left_fighting_stance );
  fill_reverse( blocking );
  fill_reverse( twohand_push );
  fill_reverse( arms_crossed );
  fill_reverse( punch_windup );
  fill_reverse( blocking );
  fill_reverse( jabkick_out );
  fill_reverse( jab_out );
  fill_reverse( reverse_out );
  fill_reverse( round_kick_outside );
  fill_reverse( round_kick_connect );
  fill_reverse( round_kick_connect2 );
  fill_reverse( elbow_connect2 );
  fill_reverse( twohand_chamber );
  fill_reverse( knee_connect );
  fill_reverse( sidekick_chamber );
  fill_reverse( sidekick_connect );
  fill_reverse( stagger_high_hit );
  fill_reverse( stagger_high_hit2 );




  /* where's the second controller?
   *
   *
   */
  /* Input device check - scan bus
   *
   *
   */
  for ( i=0; i<4; i++ ) {
    int j;
    for ( j=0; j<4; j++ ) {
      printf("Device %d %d) %d\n", i, j, maple_device_func(i,j) );

      /*** found controller (thanks Dan) ***/
      if ( maple_device_func(i,j) == MAPLE_FUNC_CONTROLLER ) {

	/*** 1st player ***/
	if ( player_count == 0 ) {
	  players[0][0] = i;
	  players[0][1] = j;
	  player_count++;
	}
	/*** just 2p for now ***/
	else {
	  players[1][0] = i;
	  players[1][1] = j;
	  player_count++;
	}
      }

    }
  }

  c = maple_create_addr( players[0][0] , players[0][1] );
  if ( player_count == 2 )  
    c2 = maple_create_addr( players[1][0] , players[1][1] ); 



  /*** Dan Potter's code ***/
  // c = maple_first_controller();
  //c2 = maple_first_mouse();

  //c2 = maple_device_func(1,0);
  //maple_second_controller();
  printf("c == %d      c2 == %d\n", c, c2 );



  
  /* MAIN LOOP:
   *  - get controller input (possibly button combos)
   *  - add it into a queue
   *  - check current controller state and history for moves
   *  - move the entity in the proper manner (interpolate)
   *  - draw it
   *
   */
  while(1) {
    int crossed = 0;

    /*** Dan Potter's code for players 1 and 2 ***/
    if (cont_get_cond(c, &cond) < 0) {
      printf("Error reading controller 1\n");
      break;
    }
    if ( player_count == 2 ) 
    if (cont_get_cond(c2, &cond2) < 0) {
      printf("Error reading controller 2\n");
      break;
    }
    



    /*** jimbo's code ***/
    /*** Log event to history ***/
    controller_history[ history_pointer++ ] = cond.buttons;
    history_pointer %= HISTORY_SIZE;

    /*** this one only logs new DIFFERENT states ***/
    if ( MOVE_AGO( 1 ) != cond.buttons ) {
      change_history[ change_pointer++ ] = cond.buttons;
      change_pointer %= HISTORY_SIZE;
    }






    /*** patch in controller logic here ***/
    if (!(cond.buttons & CONT_START))
      break;

    /*** check (both) controllers and take appropriate action ***/
    process_input2( player1, cond );
   
    if ( player_count == 2 ) 
      process_input2( player2, cond2 );
    else random_input( player2 );
 
    











    
    /* Animation timestep accounting begins here:
     *
     * eliminate old anchors block - get rid of past events
     *  (cycle the buffer past them)
     */
    /*** for macro ***/
    p = player1;
    while (  ( TIME_OF( 0 )   < 0.0  )  &&
	   (  ( TIME_OF( 1 )  < 0.0 )
	        ||
	      (
	        ( TIME_OF(1) == 0.0 ) &&
	        ( TIME_OF(2) == 0.0 )
	      )
	   )           
	     ) {
      p->lists_in_future--;
      p->future_lists[0][ p->future_pointer++ ] = 0.0;
      p->future_pointer %= FUTURE_SIZE;
    }

    /*** advance the timestep of all pending anchors ***/
    for ( i=p->future_pointer; 
	  i<p->future_pointer + p->lists_in_future;
	  i++ ) {
      crossed = 0;
      /*** if it passes the 'now' line, apply the state ***/
      if ( ( p->future_lists[0][i % FUTURE_SIZE] < timestep )  &&
	   ( p->future_lists[0][i % FUTURE_SIZE] > 0.0 )  ) { 
	crossed = 1;
	p->default_state = (int)p->future_lists[1][i % FUTURE_SIZE];
	p->fighting_state = p->default_state;
      }
      /*** timestep here ***/
      p->future_lists[0][i % FUTURE_SIZE] -= timestep;
    }
    
    /** find t parameter ***/
    t = TIME_OF( 1 ) == TIME_OF( 0 )
      ? 0.0 :
      ( -TIME_OF( 0 )  ) / (   TIME_OF( 1 ) - TIME_OF( 0 )   );
    
    /*** lock t into domain ***/
    if ( t < 0.0 ) t = 0.0;
    if ( t > 1.0 ) t = 0.0;
    
    /*** interpolate (offset for time in first slot) ***/
    if ( t != 0.0 ) {
      for ( i=3 + OFFSET_HACK; i<NUMBER_PARAMETERS + OFFSET_HACK; i++ ) {

	/* hack v0.1.5 : dont interpolate the ROTATION_OPPONENT parameter
         *  maybe just a separate field in the struct for rotation...
	 *   but then it wont be integrated into moves.
	 */
	if ( i - OFFSET_HACK != ROTATION_OPPONENT )
	p->params[i-OFFSET_HACK] = 
	  (1-t) * p->future_lists[i][p->future_pointer % FUTURE_SIZE]  + 
	  t     * p->future_lists[i][(p->future_pointer+1) % FUTURE_SIZE]  ;
      }
      /*** anchor in queue- action, so reset rotation ***/
      //     p->params[ROTATION_OPPONENT] = 0.0;
      ( (fighter_t*)(p->opponent) )->params[ROTATION_OPPONENT] = 0.0;
      
    }
    /*** no pending anchor - revert to default ***/
    else {
      //  p->fighting_state += ( p->fighting_state - p->default_state ) % 2;
    }
    p->fighting_state += ( p->fighting_state < p->default_state ) ? 1 : -1;







    /*** Extra block (as above) for second fighter ***/
    /*** for macro ***/
    p = player2;
    while (  ( TIME_OF( 0 )   < 0.0  )  &&
	   (  ( TIME_OF( 1 )  < 0.0 )
	        ||
	      (
	        ( TIME_OF(1) == 0.0 ) &&
	        ( TIME_OF(2) == 0.0 )
	      )
	   )           
	     ) {
      p->lists_in_future--;
      p->future_lists[0][ p->future_pointer++ ] = 0.0;
      p->future_pointer %= FUTURE_SIZE;
    }

    /*** advance the timestep of all pending anchors ***/
    for ( i=p->future_pointer; 
	  i<p->future_pointer + p->lists_in_future;
	  i++ ) {
      crossed = 0;
      /*** if it passes the 'now' line, apply the state ***/
      if ( ( p->future_lists[0][i % FUTURE_SIZE] < timestep )  &&
	   ( p->future_lists[0][i % FUTURE_SIZE] > 0.0 )  ) { 
	crossed = 1;
	p->default_state = (int)p->future_lists[1][i % FUTURE_SIZE];
	p->fighting_state = p->default_state;
      }
      /*** timestep here ***/
      p->future_lists[0][i % FUTURE_SIZE] -= timestep;
    }
    
    /** find t parameter ***/
    t = TIME_OF( 1 ) == TIME_OF( 0 )
      ? 0.0 :
      ( -TIME_OF( 0 )  ) / (   TIME_OF( 1 ) - TIME_OF( 0 )   );
    
    /*** lock t into domain ***/
    if ( t < 0.0 ) t = 0.0;
    if ( t > 1.0 ) t = 0.0;
    
    /*** interpolate (offset for time in first slot) ***/
    if ( t != 0.0 ) {
      for ( i=3 + OFFSET_HACK; i<NUMBER_PARAMETERS + OFFSET_HACK; i++ ) {

	/* hack v0.1.5 : dont interpolate the ROTATION_OPPONENT parameter
         *  maybe just a separate field in the struct for rotation...
	 *   but then it wont be integrated into moves.
	 */
	if ( i - OFFSET_HACK != ROTATION_OPPONENT )
	p->params[i-OFFSET_HACK] = 
	  (1-t) * p->future_lists[i][p->future_pointer % FUTURE_SIZE]  + 
	  t     * p->future_lists[i][(p->future_pointer+1) % FUTURE_SIZE]  ;
      }
      /*** anchor in queue- action, so reset rotation ***/
      // p->params[ROTATION_OPPONENT] = 0.0;
      ( (fighter_t*)(p->opponent) )->params[ROTATION_OPPONENT] = 0.0;
    }
    /*** no pending anchor - revert to default ***/
    else {
      //  p->fighting_state += ( p->fighting_state - p->default_state ) % 2;
    }
    p->fighting_state += ( p->fighting_state < p->default_state ) ? 1 : -1;






    /*** end second fighter block ***/








    /*
      p1       p2         result
      -------------------------------
      attack   --         p2 hit; add anchor fall_down; subtract energy...
      attack   block      --
      attack   counter    p1 hit; 
      
      throw    block      p2 hit
      throw    throw      --
      throw    attack     p1 hit
      throw    counter    p2 hit

      states that demand action are ATTACK, THROW


    */


    /*** collisions : rule based interaction ***/
    if ( fighting_distance < fighting_range ) {


      // if ( player1->fighting_state < FIGHT_MOVE ) 


      /*** attack vs. counter : counter wins ***/
      if ( ( player1->fighting_state == FIGHT_ATTACK ) 
	&& ( player2->fighting_state == FIGHT_COUNTER ) ) {
	/*** result: player1 hit - clear out player1's future ***/
	for ( i=0; i<FUTURE_SIZE; i++ ) {
	  player1->future_lists[0][i] = 0.0;
	}
	player1->fighting_state = FIGHT_STUNNED;
	player1->default_state = FIGHT_STUNNED;
	add_anchor( player1, 0.05, stagger_high_hit, FIGHT_STUNNED );
	add_anchor( player1, 0.35, stagger_high_hit2, FIGHT_STAND );
	printf("p2 COUNTER p1\n");
 	fighting_distance += 0.5;
      }
      /*** player converse ***/
      else if ( ( player1->fighting_state == FIGHT_COUNTER ) 
	     && ( player2->fighting_state == FIGHT_ATTACK ) ) {
	/*** result: player2 hit - clear out player2's future ***/
	for ( i=0; i<FUTURE_SIZE; i++ ) {
	  player2->future_lists[0][i] = 0.0;
	}
	player2->fighting_state = FIGHT_STUNNED;
	player2->default_state = FIGHT_STUNNED;
	add_anchor( player2, 0.05, stagger_high_hit, FIGHT_STUNNED );
	add_anchor( player2, 0.35, stagger_high_hit2, FIGHT_STAND );
	printf("p1 COUNTER p2\n", player2->fighting_state );
	fighting_distance += 0.5;
      }




      /*** attack vs. walking or standing : attack wins ***/
      else if ( ( player1->fighting_state == FIGHT_ATTACK ) 
	     && ( player2->fighting_state <= FIGHT_STAND  ) ) {
	printf("p1 ATTACK p2 %d\n", player2->fighting_state );
	/*** result: player2 hit - clear out player2's future ***/
	for ( i=0; i<FUTURE_SIZE; i++ ) {
	  player2->future_lists[0][i] = 0.0;
	}

	/*** should this happen or not? ***/
	player1->fighting_state = FIGHT_MOVE;

	player2->fighting_state -= 10; //= FIGHT_STUNNED;
	player2->default_state -= 10; //= FIGHT_STUNNED;
	add_anchor( player2, 0.05, stagger_high_hit, player2->fighting_state + 10 ); // FIGHT_STUNNED );
	add_anchor( player2, 0.35, stagger_high_hit2, player2->fighting_state + 10 ); // FIGHT_STAND );
 	fighting_distance += 0.1;
      }
      /*** player converse ***/
      else if ( ( player2->fighting_state == FIGHT_ATTACK ) 
	     && ( player1->fighting_state <= FIGHT_STAND  ) ) {
	printf("p2 ATTACK p1 %d\n", player1->fighting_state );
	/*** result: player2 hit - clear out player2's future ***/
	for ( i=0; i<FUTURE_SIZE; i++ ) {
	  player1->future_lists[0][i] = 0.0;
	}

	/*** should this happen or not? ***/
	player2->fighting_state = FIGHT_MOVE;

	player1->fighting_state -= 10; //= FIGHT_STUNNED;
	player1->default_state -= 10; //= FIGHT_STUNNED;
	add_anchor( player1, 0.05, stagger_high_hit, player1->fighting_state + 10 ); // FIGHT_STUNNED );
	add_anchor( player1, 0.35, stagger_high_hit2, player1->fighting_state + 10 ); // FIGHT_STAND );
 	fighting_distance += 0.1;
      }















      //     printf(" %d %d \n", player1->fighting_state, player2->fighting_state );




    } /*** out of range (fighting_distance ) ***/
 
  



    










    /* Animation done.
     *  Begin frame :
     *   display code below
     */
    glKosBeginFrame();
    
    /*** Draw in space ***/
    glLoadIdentity();
    glPushMatrix();

      /*** stamina indicator1 - vertically along left of screen ***/
      glBegin(GL_QUADS);
      glVertex3f( -4.0, 0.0, -5.0 );
      glVertex3f( -4.0, 3.0 * ((double)player1->fighting_state/(double)150), -5.0 );
      glVertex3f( -3.0, 3.0 * ((double)player1->fighting_state/(double)150), -5.0 );
      glVertex3f( -3.0, 0.0, -5.0 );
      glEnd();



      /*** stamina indicator2 - vertically along right of screen ***/
      glBegin(GL_QUADS);
      glVertex3f( 3.0, 0.0, -5.0 );
      glVertex3f( 3.0, 3.0 * ((double)player2->fighting_state/(double)150), -5.0 );
      glVertex3f( 4.0, 3.0 * ((double)player2->fighting_state/(double)150), -5.0 );
      glVertex3f( 4.0, 0.0, -5.0 );
      glEnd();


      

      glTranslatef(0.0f, 0.0f, -20.0f );




      /*** buffer indicator ***/
      glBegin(GL_QUADS);  
        glVertex3f(-10.0f, 16.0f, -15.0f);
	glVertex3f( -10.0 + ((double)player1->lists_in_future/(double)FUTURE_SIZE) * 20.0f, 16.0f, -15.0f);
	glVertex3f( -10.0 + ((double)player1->lists_in_future/(double)FUTURE_SIZE) * 20.0f, 14.0f, -15.0f);
	glVertex3f(-10.0f, 14.0f, -15.0f);
      glEnd();

      /*** buffer indicator2 ***/
      glBegin(GL_QUADS);  
        glVertex3f(-10.0f, 14.0f, -15.0f);
	glVertex3f( -10.0 + ((double)player2->lists_in_future/(double)FUTURE_SIZE) * 20.0f, 14.0f, -15.0f);
	glVertex3f( -10.0 + ((double)player2->lists_in_future/(double)FUTURE_SIZE) * 20.0f, 12.0f, -15.0f);
	glVertex3f(-10.0f, 12.0f, -15.0f);
      glEnd();



      /*** free "camera" ***/
      glRotatef( view_rot, 0.0, 1.0, 0.0 );
      glRotatef( view_roty, 1.0, 0.0, 0.0 );



      /* floor : rotates about center of two fighters to give the illusion
       *  of motion.  in reality, the two fighters are each being rotated 
       *   by their view_angle, which the opponent has control over.
       *    relativity: press up to strafe around somebody, they will appear
       *     to rotate the opposite direction of your strafe with a camera
       *      on the perpendicular bisector.
       */
      glPushMatrix();
      /*** v0.1.5: move it too ***/
      glTranslatef( -fighting_displacement[0], 0.0, fighting_displacement[1] );
      glRotatef( -fighting_rotation, 0.0, 1.0, 0.0 );

      //  glTranslatef( -fighting_distance, 0.0, 0.0 );
      //  glRotatef( -90.0 + player2->params[ ROTATION_OPPONENT ], 0.0, 1.0, 0.0 );

      //      glTranslatef( fighting_displacement[1], 0.0, fighting_displacement[0] );
      glBegin(GL_QUADS);  
        glVertex3f(-10.0f, -7.0f, -10.0f);
	glVertex3f( 10.0f, -7.0f, -10.0f);
	glVertex3f( 10.0f, -7.0f,  10.0f);
	glVertex3f(-10.0f, -7.0f,  10.0f);
      glEnd();
      glPopMatrix();




      /* players 1 and 2: each fighter is individually rotated about the other 
       *  players' params[ ROTATION_OPPONENT ] : manipulated by the up and 
       *   down buttons.
       */
      glPushMatrix();
        glTranslatef( -fighting_distance, -3.0, 0.0 );
        glRotatef( -90.0 + player2->params[ ROTATION_OPPONENT ]
		   //      + player2->params[ ROTATION_OPPONENT ]
		   , 0.0, 1.0, 0.0 );
	display_entity( player1->params );
      glPopMatrix();
      /*** player 2 ***/
      glPushMatrix();
        glTranslatef( fighting_distance, -3.0, 0.0 );
        glRotatef( 90.0 + player1->params[ ROTATION_OPPONENT ]
		   //     + player1->params[ ROTATION_OPPONENT ]
		   , 0.0, 1.0, 0.0 );
	display_entity( player2->params );
      glPopMatrix();





    glPopMatrix();
    
    /* Finish the frame */
    glKosFinishFrame();
  } /*** end main loop ***/
  
  return 0;
}


